import csv
import datetime
import numpy as np
import os
import sys


###################################################################
###################################################################
### Ersteller: Das Corona Blog Team
###################################################################
### Starte mit
### python age_grabber_v1.py JAHR MONAT TAG DAUER
###################################################################
###################################################################

# Startdatum und Dauer von Kommandozeilenargumenten
start_date = datetime.date(int(sys.argv[1]), int(sys.argv[2]), int(sys.argv[3]))
duration = int(sys.argv[4])

# Auszuwertende Spalten
csv_val = {
    "Impfquote_12bis17_voll" : None,
    "Impfquote_12bis17_boost" : None,
    "Impfquote_18bis59_voll" : None,
    "Impfquote_18bis59_boost" : None,
    "Impfquote_60plus_voll" : None,
    "Impfquote_60plus_boost" : None,
    "Impfquote_18plus_min1" : None,
    "Impfquote_18plus_voll" : None,
}

print(f"Berechne für den Zeitraum vom {start_date} bis {start_date + datetime.timedelta(days=duration)}:")

# Pfad der zur aktuellen Datei
path = os.path.dirname(os.path.realpath(__file__))

# Auslesen der csv Dateien
for element in csv_val.keys():
    d = start_date
    temp_list = []
    for x in range(duration):
        filename = path + "/Archiv/" + str(d.year) + "-" + \
            '{:02d}'.format(d.month) + "-" \
            + '{:02d}'.format(d.day) +  \
            "_Deutschland_Impfquoten_COVID-19.csv"
        try:
            with open(filename, newline='') as csvfile:
                content = csv.reader(csvfile, delimiter=',', quotechar='|')
                rows = list(content)

                # find number of column that holds our string
                n_col = 999
                i = 0
                for title in rows[0]:
                    if title == element:
                        n_col = i
                        temp_list.append(float(rows[1][n_col]))
                    i += 1
        
        # Wenn eine Datei fehlt (z.B. Sonntag) ignoriere das
        except OSError:
            pass

        d += datetime.timedelta(days=1)

    csv_val[element] = np.array(temp_list)
    print(f"Mittelwert {element}: {np.mean(csv_val[element]):.2f}")



# Jetzt werden verschiedene Angaben berechnet
# die "XXX_ungeimpft" Angaben enthalten die Ungeimpften und nur Erstgeimpften
calc_val = {
        "Impfquote_12bis17_ungeimpft" : None, "rki_quote_12bis17_voll" : None, "rki_quote_12bis17_boost" : None,
        "Impfquote_18bis59_ungeimpft" : None, "rki_quote_18bis59_voll" : None, "rki_quote_18bis59_boost" : None,
        "Impfquote_60plus_ungeimpft" : None, "rki_quote_60plus_voll" : None, "rki_quote_60plus_boost" : None,
        "Quote_18plus_erstgeimpft" : None,
        "rki_quote_18bis59_voll_max" : None, "rki_quote_18bis59_boost_max" : None,
        "rki_quote_60plus_voll_max" : None, "rki_quote_60plus_boost_max" : None,
}


# Anteil (an der Bevölkerung) an Personen im Alter 18+, die nur erstgeimpft sind
calc_val["Quote_18plus_erstgeimpft"] = csv_val["Impfquote_18plus_min1"] - csv_val["Impfquote_18plus_voll"]

# 12 - 17
calc_val["Impfquote_12bis17_ungeimpft"] = np.full(csv_val["Impfquote_12bis17_voll"].size, 100) - csv_val["Impfquote_12bis17_voll"]
calc_val["rki_quote_12bis17_voll"] = (csv_val["Impfquote_12bis17_voll"] - csv_val["Impfquote_12bis17_boost"]) / \
                                    (csv_val["Impfquote_12bis17_voll"] - csv_val["Impfquote_12bis17_boost"] + \
                                         calc_val["Impfquote_12bis17_ungeimpft"]) * 100.
calc_val["rki_quote_12bis17_boost"] = csv_val["Impfquote_12bis17_boost"] / \
                                    (csv_val["Impfquote_12bis17_boost"] + calc_val["Impfquote_12bis17_ungeimpft"]) * 100.

# 18 - 59
calc_val["Impfquote_18bis59_ungeimpft"] = np.full(csv_val["Impfquote_18bis59_voll"].size, 100) - csv_val["Impfquote_18bis59_voll"]
calc_val["rki_quote_18bis59_voll"] = (csv_val["Impfquote_18bis59_voll"] - csv_val["Impfquote_18bis59_boost"]) / \
                                    (csv_val["Impfquote_18bis59_voll"] - csv_val["Impfquote_18bis59_boost"] + \
                                         calc_val["Impfquote_18bis59_ungeimpft"]) * 100.
calc_val["rki_quote_18bis59_boost"] = csv_val["Impfquote_18bis59_boost"] / \
                                    (csv_val["Impfquote_18bis59_boost"] + calc_val["Impfquote_18bis59_ungeimpft"]) * 100.
# Schätzung --> Alle Erstgeimpften in Gruppe 18 - 59
calc_val["rki_quote_18bis59_voll_max"] = (csv_val["Impfquote_18bis59_voll"] - csv_val["Impfquote_18bis59_boost"]) / \
                                    (csv_val["Impfquote_18bis59_voll"] - csv_val["Impfquote_18bis59_boost"] + \
                                         calc_val["Impfquote_18bis59_ungeimpft"] - calc_val["Quote_18plus_erstgeimpft"] ) * 100.
calc_val["rki_quote_18bis59_boost_max"] = csv_val["Impfquote_18bis59_boost"] / \
                                    (csv_val["Impfquote_18bis59_boost"] + calc_val["Impfquote_18bis59_ungeimpft"] - \
                                         calc_val["Quote_18plus_erstgeimpft"] ) * 100.

# 60+
calc_val["Impfquote_60plus_ungeimpft"] = np.full(csv_val["Impfquote_60plus_voll"].size, 100) - csv_val["Impfquote_60plus_voll"]
calc_val["rki_quote_60plus_voll"] = (csv_val["Impfquote_60plus_voll"] - csv_val["Impfquote_60plus_boost"]) / \
                                    (csv_val["Impfquote_60plus_voll"] - csv_val["Impfquote_60plus_boost"] + \
                                         calc_val["Impfquote_60plus_ungeimpft"]) * 100.
calc_val["rki_quote_60plus_boost"] = csv_val["Impfquote_60plus_boost"] / \
                                    (csv_val["Impfquote_60plus_boost"] + calc_val["Impfquote_60plus_ungeimpft"]) * 100.
# Schätzung --> Alle Erstgeimpften in Gruppe 60+
calc_val["rki_quote_60plus_voll_max"] = (csv_val["Impfquote_60plus_voll"] - csv_val["Impfquote_60plus_boost"]) / \
                                    (csv_val["Impfquote_60plus_voll"] - csv_val["Impfquote_60plus_boost"] + \
                                         calc_val["Impfquote_60plus_ungeimpft"] - calc_val["Quote_18plus_erstgeimpft"]) * 100.
calc_val["rki_quote_60plus_boost_max"] = csv_val["Impfquote_60plus_boost"] / \
                                    (csv_val["Impfquote_60plus_boost"] + calc_val["Impfquote_60plus_ungeimpft"] -\
                                         calc_val["Quote_18plus_erstgeimpft"]) * 100.


print("\nDie berechneten Werte sind:")
for key, calc_val_item in calc_val.items():
    print(f"Mittelwert {key}: {np.mean(calc_val_item):.2f}")
